package com.bytesw.bytebot.service;

import com.bytesw.bytebot.bean.AgentBean;
import com.bytesw.bytebot.bean.DeploymentChannelBean;
import com.bytesw.bytebot.bean.DeploymentChannelParamValueBean;
import com.bytesw.bytebot.model.*;
import com.bytesw.bytebot.model.enums.AgentTypeEnum;
import com.bytesw.bytebot.model.enums.StatusEnum;
import com.bytesw.bytebot.repository.*;
import com.bytesw.xdf.sql.beans.Pagination;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import javax.swing.text.html.Option;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

@Service
@Log4j2
public class AgentService extends CustomPaginationService<Agent> {

    @Autowired
    private AgentRepository agentRepository;

    @Autowired
    private CountryRepository countryRepository;

    @Autowired
    private DeploymentChannelRepository deploymentChannelRepository;

    @Autowired
    private DeploymentChannelParamValueRepository deploymentChannelParamValueRepository;

    @Autowired
    private ChannelRepository channelRepository;

    @Autowired
    private ChannelParamRepository channelParamRepository;

    @Transactional(readOnly = true, propagation = Propagation.NOT_SUPPORTED)
    public void searchByPagination(Pagination<AgentBean> pagination) {
        if (pagination.getItemsPerPage() == 0) {
            pagination.setItemsPerPage(10);
        }

        Sort sort = null;
        if (pagination.getSortFields() != null) {
            sort = createSort(pagination);
        }

        Pageable pageable = null;
        if (sort != null) {
            pageable = PageRequest.of(pagination.getCurrentPage(), pagination.getItemsPerPage(), sort);
        } else {
            pageable = PageRequest.of(pagination.getCurrentPage(), pagination.getItemsPerPage());
        }

        Specification<Agent> specification = (Specification<Agent>) (root, query, criteriaBuilder)
                -> criteriaBuilder.notEqual(root.get("status"), StatusEnum.DELETED);

        if (pagination.getFilterExpression() != null && pagination.getFilterExpression().length() > 0) {
            specification = createSpecification(pagination.getFilterExpression()).and(specification);
        }

        Page<Agent> page = agentRepository.findAll(specification, pageable);

        pagination.setTotalItems(page.getTotalElements());
        pagination.setTotalPages(page.getTotalPages());

        // Clone to array
        List<AgentBean> list = new ArrayList<>();

        for (Agent agent : page.getContent()) {
            list.add(AgentBean.miniClone(agent));
        }

        pagination.setData(list);
    }

    @Transactional(propagation = Propagation.REQUIRED)
    public void save(AgentBean agent) {

        boolean isModify = true;

        Agent agentBD = null;

        if (agent.getId() != null) {
            Optional<Agent> agentFound = agentRepository.findById(agent.getId());

            if (agentFound.isPresent()) {
                agentBD = agentFound.get();
            }
        }

        if (agent == null) {
            agentBD = new Agent();
            agentBD.setStatus(StatusEnum.ACTIVE);

            isModify = false;
        }

        // General Information

        agentBD.setName(agent.getName());
        agentBD.setDescription(agent.getDescription());
        agentBD.setLanguage(agent.getLanguage());
        agentBD.setTimezone(agent.getTimezone());
        agentBD.setVersion(agent.getVersion());
        agentBD.setType(AgentTypeEnum.fromString(agent.getType()));
        agentBD.setAvatar(agent.getAvatar());

        if (agent.getCountryId() != null) {
            Optional<Country> countryFound = countryRepository.findById(agent.getCountryId());

            if (countryFound.isPresent()) {
                agentBD.setCountry(countryFound.get());
            }
        } else {
            agentBD.setCountry(null);
        }

        // Frequent questions


        // Deployment channels
        if (agent.getDeploymentChannels() == null) {
            agent.setDeploymentChannels(new ArrayList<>());
        }

        if (agentBD.getDeploymentChannels() != null) {
            for (DeploymentChannel deploymentChannelBD : agentBD.getDeploymentChannels()) {
                boolean found = false;

                for (DeploymentChannelBean deploymentChannelBean : agent.getDeploymentChannels()) {
                    if (deploymentChannelBean.getId() != null && deploymentChannelBD.getId().compareTo(deploymentChannelBean.getId()) == 0) {
                        found = true;
                        break;
                    }
                }

                if (!found) {
                    deploymentChannelParamValueRepository.deleteAll(deploymentChannelBD.getParameters());
                    deploymentChannelRepository.delete(deploymentChannelBD);
                }
            }
        }

        for (DeploymentChannelBean deploymentChannelBean : agent.getDeploymentChannels()) {
            DeploymentChannel deploymentChannelBD = null;

            if (deploymentChannelBean.getId() != null) {
                Optional<DeploymentChannel> deploymentChannelFound = deploymentChannelRepository.findById(deploymentChannelBean.getId());

                if (deploymentChannelFound.isPresent()) {
                    deploymentChannelBD = deploymentChannelFound.get();
                }
            }

            if (deploymentChannelBD == null) {
                deploymentChannelBD = new DeploymentChannel();
            }

            deploymentChannelBD.setAgent(agentBD);
            deploymentChannelBD.setName(deploymentChannelBean.getName());
            deploymentChannelBD.setStatus(StatusEnum.fromString(deploymentChannelBean.getStatus()));

            if (deploymentChannelBean.getChannelId() != null) {
                Optional<Channel> channelFound = channelRepository.findById(deploymentChannelBean.getChannelId());

                if (channelFound.isPresent()) {
                    deploymentChannelBD.setChannel(channelFound.get());
                }
            } else {
                deploymentChannelBD.setChannel(null);
            }

            // Deployment Channel parameters

            if (deploymentChannelBean.getParameters() == null) {
                deploymentChannelBean.setParameters(new ArrayList<>());
            }

            if (deploymentChannelBD.getParameters() != null) {
                for (DeploymentChannelParamValue deploymentChannelParamValueBD : deploymentChannelBD.getParameters()) {
                    boolean found = false;

                    for (DeploymentChannelParamValueBean deploymentChannelParamValueBean : deploymentChannelBean.getParameters()) {
                        if (deploymentChannelParamValueBean.getId() != null &&
                                deploymentChannelParamValueBD.getId().compareTo(deploymentChannelParamValueBean.getId()) == 0) {
                            found = true;
                            break;
                        }
                    }

                    if (!found) {
                        deploymentChannelParamValueRepository.delete(deploymentChannelParamValueBD);
                    }
                }
            }

            for (DeploymentChannelParamValueBean deploymentChannelParamValueBean : deploymentChannelBean.getParameters()) {
                DeploymentChannelParamValue deploymentChannelParamValue = null;

                if (deploymentChannelParamValueBean.getId() != null) {
                    Optional<DeploymentChannelParamValue> deploymentChannelParamValueFound = deploymentChannelParamValueRepository
                            .findById(deploymentChannelParamValueBean.getId());

                    if (deploymentChannelParamValueFound.isPresent()) {
                        deploymentChannelParamValue = deploymentChannelParamValueFound.get();
                    }
                }

                if (deploymentChannelParamValue == null) {
                    deploymentChannelParamValue = new DeploymentChannelParamValue();
                }

                deploymentChannelParamValue.setDeploymentChannel(deploymentChannelBD);
                deploymentChannelParamValue.setValue(deploymentChannelParamValueBean.getValue());

                if (deploymentChannelParamValueBean.getChannelParamId() != null) {
                    Optional<ChannelParam> channelParamFound = channelParamRepository.findById(deploymentChannelParamValueBean.getChannelParamId());

                    if (channelParamFound.isPresent()) {
                        deploymentChannelParamValue.setParameter(channelParamFound.get());
                    }
                }

                deploymentChannelParamValue.setParameter(null);
            }

        }

    }

    @Transactional(propagation = Propagation.NOT_SUPPORTED)
    public AgentBean getAgent(Long id) {
        AgentBean bean = null;

        Optional<Agent> agentFound = agentRepository.findById(id);

        if (agentFound.isPresent()) {
            Agent agent = agentFound.get();

            bean = new AgentBean();

            bean.setId(agent.getId());
            bean.setName(agent.getName());
            bean.setDescription(agent.getDescription());
            bean.setVersion(agent.getVersion());
            bean.setLanguage(agent.getLanguage());
            bean.setTimezone(agent.getTimezone());
            bean.setType(agent.getType().getName());
            bean.setStatus(agent.getStatus().getName());
            bean.setAvatar(agent.getAvatar());

            if (agent.getCountry() != null) {
                bean.setCountryName(agent.getCountry().getName());
                bean.setCountryId(agent.getCountry().getId());
            }

            bean.setDeploymentChannels(new ArrayList<>());

            // Deployment channel

            for (DeploymentChannel deploymentChannel : agent.getDeploymentChannels()) {
                DeploymentChannelBean deploymentChannelBean = new DeploymentChannelBean();
                deploymentChannelBean.setId(deploymentChannel.getId());
                deploymentChannelBean.setName(deploymentChannel.getName());
                deploymentChannelBean.setStatus(deploymentChannel.getStatus().getName());
                deploymentChannelBean.setParameters(new ArrayList<>());

                if (deploymentChannel.getChannel() != null) {
                    deploymentChannelBean.setChannelId(deploymentChannel.getChannel().getId());
                    deploymentChannelBean.setChannelName(deploymentChannel.getChannel().getName());
                }

                //Parameters

                for (DeploymentChannelParamValue parameter : deploymentChannel.getParameters()) {
                    DeploymentChannelParamValueBean parameterBean = new DeploymentChannelParamValueBean();
                    parameterBean.setId(parameter.getId());
                    parameterBean.setValue(parameter.getValue());

                    if (parameter.getParameter() != null) {
                        parameterBean.setChannelParamId(parameter.getParameter().getId());
                    }

                    deploymentChannelBean.getParameters().add(parameterBean);
                }

                bean.getDeploymentChannels().add(deploymentChannelBean);
            }
        }

        return bean;
    }

    @Transactional(propagation = Propagation.REQUIRED)
    public boolean delete(Long id) {
        boolean isValid = true;

        if (id != null) {
            Optional<Agent> agentFound = agentRepository.findById(id);

            if (agentFound.isPresent()) {
                Agent agent = agentFound.get();
                agent.setStatus(StatusEnum.DELETED);

                agentRepository.save(agent);
            }
        }

        return isValid;
    }

}
